! Server routines for browse windows

  MEMBER

_ABCDllMode_  EQUATE(0)
_ABCLinkMode_ EQUATE(1)

  map
    CasedValue(STRING FieldName,*? Field,? FieldValue),STRING
  end

  include('AlBrw.inc'),ONCE


AlterRangeBrowseClass.Init     PROCEDURE(SIGNED ListBox,*STRING Posit,VIEW V,QUEUE Q,RelationManager RM,WindowManager WM)
! init and set the defaults

  code

  parent.Init(ListBox, Posit, V, Q, RM, WM)
  Self.SelectListOnReset = true     ! select list box on reset actions
  Self.RangeLimitCondition = '='

  return
! end Init -----------------------------------------------------------------------------

AlterRangeBrowseClass.RangelimitActive        procedure() !byte
! return true if a range limit is active, if LimitType is anyvalue other than None
! a range list is active

  code

  return(Self.Order.LimitType)
! end RangeLimitActive ------------------------------------------------------------------

AlterRangeBrowseClass.ResetRangeLimit         procedure() !virtual
! clear the limit

  code

  if (Self.Order.LimitType <> Limit:None) 
    ! kill the buffered pairs class and reset the range
    Self.Order.RangeList.Kill()
    Self.ResetRange()
  end ! if

  return
! end ResetRangeLimit -------------------------------------------------------------

AlterRangeBrowseClass.ResetRangeLimit         procedure(*? FieldName) !virtual
! reset range limit to current type limit,
! field should be primed before calling

  code

  Self.ResetRangeLimit()
  Self.Order.RangeList.Init()
  Self.AddRange(FieldName)
  Self.ResetRange(Limit:Current)

  return
! end ResetRangeLimit ---------------------------------------------------------------

AlterRangeBrowseClass.ResetRangeLimit   procedure(*? FieldName, *? StartField) !virtual
! reset range limit to single value type

  code

  ! if a range limit is active clear it first
  Self.ResetRangeLimit()
  
  Self.Order.RangeList.Init()
  Self.AddRange(FieldName, StartField)
  Self.ResetRange(Limit:Single)

  return
! end ResetRangeLimit --------------------------------------------------------------

AlterRangeBrowseClass.ResetRangeLimit   procedure(*? FieldName, *? StartField, *? StopField) !virtual
! reset range limit to pair type

  code

  ! if a range limit is active clear it first
  Self.ResetRangeLimit()
  
  Self.Order.RangeList.Init()
  Self.AddRange(FieldName, StartField, StopField)
  Self.ResetRange(Limit:Pair)

  return
! end ResetRangeLimit --------------------------------------------------------------

AlterRangeBrowseClass.SetRangeControl  procedure(unsigned ClearControl)
! assign the controls used by the window

  code

  Self.ClearRangeControl = ClearControl

  return
! end SetRangeControls -----------------------------------------------------------------

AlterRangeBrowseClass.SetSelectList  procedure(byte OnOff = true)
! toggle the flag, true then list is selected after reset range action

  code

  Self.SelectListOnReset = OnOff

  return
! end SetSelectList ----------------------------------------------------------------

AlterRangeBrowseClass.SetClearCode            procedure(long K = CtrlR)   ! default code
! set the clear key code

  code

  Self.ClearRangeCode = K
  Self.ListControl{prop:alrt, 255} = K

  return
! end SetClearCode -----------------------------------------------------------------
AlterRangeBrowseClass.SetRangeCondition      procedure(string Cond)

  code

  Self.RangeLimitCondition = Cond

  return
! end SetSingleLimitType ---------------------------------------------------------

AlterRangeBrowseClass.GetRangeCondition      procedure() !string
! return the current value

  code
  return(Self.RangeLimitCondition)
! end GetSingleLimitType ----------------------------------------------------------

AlterRangeBrowseClass.ApplyFilter   procedure ! virtual
! alters the condition used in current and single value range limits.
! set the SingleLimitType to >, <, and this method will add tat to the
! range limit condition.  If set to = then default is used.

  code

  case Self.Order.LimitType
    of Limit:Single
    orof Limit:Current
      if (Self.RangeLimitCondition <> '=')
        Self.AdjustSingleFilter()
      else
        parent.ApplyFilter()
      end
    of Limit:Pair
      if (Self.RangeLimitCondition <> '=')
        Self.AdjustPairFilter()
      else
        parent.ApplyFilter()
      end
  else
    parent.ApplyFilter()
  end

  return
! end ApplyFilter ------------------------------------------------------------------

AlterRangeBrowseClass.GetFreeElementPosition  PROCEDURE  !BYTE,PROTECTED,VIRTUAL
! method is not really needed is only included to correct a bug in the view manager class
! when using single component key and a pair type limit

ReturnValue    byte,auto

  code

  ReturnValue = parent.GetFreeElementPosition()
  if (ReturnValue = 0) and ( (Self.Order.LimitType = Limit:Pair) and (Self.Primary.Me.GetComponents(Self.Order.MainKey) = 1) )
    ReturnValue = 1
  end

  return(ReturnValue)

! end GetFreeElementPosition ------------------------------------------------------

AlterRangeBrowseClass.TakeEvent    procedure
! check for clear control accepted, if control is in use

  code

  if (Self.ClearRangeControl) and (accepted() = Self.ClearRangeControl)
    ! is there a control and was it accepted
    Self.ResetRangeLimit()
  else
    parent.TakeEvent()
  end

  return
! end TakeEvent ----------------------------------------------------------------------

AlterRangeBrowseClass.TakeKey        PROCEDURE  !BYTE,PROC,VIRTUAL 
! check for alert key, used to clear the range, if in use

ReturnValue         byte(true)

  code

  ! is derived class interested
  if (event() = Event:ALertKey)
    if (keycode() = Self.ClearRangeCode)
      Self.ResetRangeLimit()
      ReturnValue = false
    end
  end

  if (ReturnValue)
    ReturnValue = parent.TakeKey()
  end

  return(ReturnValue)
! end TakeKey ---------------------------------------------------------------------------

AlterRangeBrowseClass.UpdateWindow   procedure !VIRTUAL
! update controls

  code

  parent.UpdateWindow()
  ! update parent and set Clear Range Control
  if (Self.Order.LimitType <> Limit:None)
    if (Self.ClearRangeControl)
      Self.ClearRangeControl{prop:disable} = false
    end ! if
  else
    if (Self.ClearRangeControl)
      Self.ClearRangeControl{prop:disable} = true
    end ! if
  end ! if

  return
! end UpdateWindow ------------------------------------------------------------------

! -------------------------------------------------
! private methods
! -------------------------------------------------

AlterRangeBrowseClass.ResetRange   procedure(byte LimitType = Limit:None) ! private
! reset the range limit

  code

  Self.Order.LimitType = LimitType
  put(Self.Order)                   ! store it
  assert(~errorcode())              ! if error we have problems

  Self.ResetSort(true)              ! reset the sort to display the new limits
  if (Self.SelectListOnReset)
    select(Self.ListControl)        ! select the list box
  end ! if

  return
! end ResetRange --------------------------------------------------------------------

AlterRangeBrowseClass.AdjustSingleFilter  procedure() ! private
! change the condition of the filter  method is called when Limit:Single or Current type
! range is active and the SingleLimitType property is not equal to '='

NumberRecs   long,auto         ! number records in rangelist
i            byte,auto         ! local counter
FieldName    cstring(500)      ! local temp values
RangeFilter  cstring(5000)

  code

  ! loop thru records and add in range limits
  NumberRecs = records(Self.Order.RangeList.List)
  loop i = 1 to NumberRecs
    get(Self.Order.RangeList.List,i)
    FieldName = SELF.Primary.Me.GetFieldName(SELF.Order.MainKey,i)
    if (i = NumberRecs)
      ! if at last element then use the condition set in the class property 
      RangeFilter = RangeFilter & choose(i = 1,'',' AND ') & FieldName & ' ' & Self.RangeLimitCondition & '= ' & CasedValue(FieldName,SELF.Order.RangeList.List.Left,SELF.Order.RangeList.List.Right)
    else
      ! not last then add as normal
      RangeFilter = RangeFilter & choose(i = 1,'',' AND ') & FieldName & ' = ' & CasedValue(FieldName,SELF.Order.RangeList.List.Left,SELF.Order.RangeList.List.Right)
    end ! if
  end ! loop

  Self.AddFilterCriteria(RangeFilter)

  return
! AdjustSingleFilter ----------------------------------------------------------------

AlterRangeBrowseClass.AdjustPairFilter  procedure() ! private
! change the condition of the filter, method is called when Limit:Pair type
! range is active and the SingleLimitType property is not equal to '='

NumberRecs   long,auto         ! number records in rangelist
i            byte,auto         ! local counter
FieldName    cstring(500)      ! local temp values
RangeFilter  cstring(5000)

  code

  ! get the number of records and loop one less
  NumberRecs = records(Self.Order.RangeList.List)
  loop i = 1 to NumberRecs - 1
    get(Self.Order.RangeList.List,i)
    FieldName = SELF.Primary.Me.GetFieldName(SELF.Order.MainKey,I)
    case i
      ! if 1 less then at next to last element
      of NumberRecs - 1
        ! if the right side is not blank then add normally, if empty skip it
        if (Self.Order.RangeList.List.Right)
          RangeFilter = RangeFilter & choose(RangeFilter = '','',' AND ') & FieldName & ' >= ' & CasedValue(FieldName,SELF.Order.RangeList.List.Left,SELF.Order.RangeList.List.Right)
        end ! if
    else ! for case i
      RangeFilter = RangeFilter & choose(I = 1,'',' AND ') & FieldName & ' = ' & CasedValue(FieldName,SELF.Order.RangeList.List.Left,SELF.Order.RangeList.List.Right)
    end ! case 
  end ! loop
  ! get the last element, if not empty then add norm, if the right side is empty then skip
  get(Self.Order.RangeList.List,NumberRecs)
  if (Self.Order.RangeList.List.Right)
    RangeFilter = RangeFilter & choose(RangeFilter = '','',' AND ') & FieldName & ' <<= ' & CasedValue(FieldName,SELF.Order.RangeList.List.Left,SELF.Order.RangeList.List.Right)
  end ! if

  Self.AddFilterCriteria(RangeFilter)

  return
! AdjustPairFilter ----------------------------------------------------------------

AlterRangeBrowseClass.AddFilterCriteria  procedure(*cstring Rf) ! private
! add filter criteria to view property after range limits
! only called when RangeLimitCondition is not equal to '='

i   byte,auto

  code

  ! add in filter criteria
  if ~(Self.Order.Filter &= NULL)
    loop i = 1 to records(Self.Order.Filter)
      get(Self.Order.Filter,I)
      Rf = Rf & choose(Rf = '','(',' AND (') & Self.Order.Filter.Filter & ')'
    end ! loop 
  end ! if

  ! set the filter property and test for error
  Self.View{prop:filter} = Rf
  if errorcode()
    Self.Primary.Me.Throw(Msg:ViewOpenFailed)
  end ! if

  return
! AddFilterCriteria ------------------------------------------------------------------

! upper added to the fields, from abfile.clw, not part of a class so I copied
CasedValue PROCEDURE(STRING FieldName,*? Field,? FieldValue)
TValue CSTRING(2000),AUTO
Value CSTRING(2000),AUTO
EQ    BYTE,AUTO
I     UNSIGNED,AUTO
J     UNSIGNED(2)
  CODE
    IF IsString(Field)
      TValue = FieldValue
      Value[1] = ''''
      LOOP I = 1 TO LEN(TValue)
        Value[J] = TValue[I]
        J += 1
        IF TValue[I] = ''''        ! Double up for parsing purposes
          Value[J] = TValue[I]
          J += 1
        END
      END
      Value[J :J+1] = '''<0>'
      RETURN CHOOSE(INSTRING('UPPER(',FieldName,1,1) <> 0,'UPPER('&Value&')',Value)
    ELSE
      RETURN FieldValue
    END
